@@ -75,6 +75,7 @@ hitori_create_interface (Hitori *hitori)
 	GtkBuilder *builder;
 	GtkStyleContext *style_context;
 	const PangoFontDescription *font;
+	GMenuModel *app_menu, *win_menu;  /* owned */
 
 	builder = gtk_builder_new ();
 
@@ -106,11 +107,15 @@ hitori_create_interface (Hitori *hitori)
 	hitori->timer_label = GTK_LABEL (gtk_builder_get_object (builder, "hitori_timer"));
 
 	/* Set up the menus (application and window). */
+	app_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "app_menu"));
 	g_action_map_add_action_entries (G_ACTION_MAP (hitori), app_entries, G_N_ELEMENTS (app_entries), hitori);
-	gtk_application_set_app_menu (GTK_APPLICATION (hitori), G_MENU_MODEL (gtk_builder_get_object (builder, "app_menu")));
+	gtk_application_set_app_menu (GTK_APPLICATION (hitori), app_menu);
+	g_object_unref (app_menu);
 
+	win_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "win_menu"));
 	g_action_map_add_action_entries (G_ACTION_MAP (hitori->window), win_entries, G_N_ELEMENTS (win_entries), hitori);
-	gtk_application_set_menubar (GTK_APPLICATION (hitori), G_MENU_MODEL (gtk_builder_get_object (builder, "win_menu")));
+	gtk_application_set_menubar (GTK_APPLICATION (hitori), win_menu);
+	g_object_unref (win_menu);
 
 	hitori->undo_action = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (hitori->window), "undo"));
 	hitori->redo_action = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (hitori->window), "redo"));
@@ -316,8 +321,6 @@ hitori_button_release_cb (GtkWidget *drawing_area, GdkEventButton *event, Hitori
 	/* Clamp the width/height to the minimum */
 	if (height < width)
 		width = height;
-	else if (width < height)
-		height = width;
 
 	cell_size = width / hitori->board_size;
 
@@ -356,10 +359,20 @@ hitori_button_release_cb (GtkWidget *drawing_area, GdkEventButton *event, Hitori
 
 	hitori->made_a_move = TRUE;
 
-	if (hitori->undo_stack != NULL)
+	if (hitori->undo_stack != NULL) {
+		HitoriUndo *i, *next = NULL;
+
+		/* Free the redo stack after this point. */
+		for (i = hitori->undo_stack->redo; i != NULL; i = next) {
+			next = i->redo;
+			g_free (i);
+		}
+
 		hitori->undo_stack->redo = undo;
+	}
 	hitori->undo_stack = undo;
 	g_simple_action_set_enabled (hitori->undo_action, TRUE);
+	g_simple_action_set_enabled (hitori->redo_action, FALSE);
 
 	/* Stop any current hints */
 	hitori_cancel_hinting (hitori);
